#include "class(y).ch"
#include "mrdebug.ch"
#include "inkey.ch"

/*Ŀ
      ClassaBrowse                                                         
    PurposeArray List Class Base version (does not assume column)          
  Date-Time08-12-93      12:43:23pm                                        
  CopyrightL&W Computer Services Ltd.                                      
     AuthorIan Day                                                         
  */
CREATE CLASS abBrowse FROM tBrowse

PROTECTED:
   METHOD Init

EXPORT:
   VAR aArray                 TYPE ARRAY

   VAR nElement               TYPE NUMERIC

   VAR TopArrow, BottomArrow  TYPE ARRAY

   VAR bFinder                TYPE BLOCK

   VAR cSearch                TYPE CHARACTER

   VAR lSearchable            TYPE LOGICAL

   VAR nCursorRow, nCursorCol TYPE INT

   METHOD Skipper, DisplayArrows, Handler, Settle, Stabilize, ForceStable,;
          DoCursor

END CLASS

/*Ŀ
     MethodInit() --> self                                                 
    PurposeInitialise object                                               
  Date-Time08-12-93      12:47:57pm                                        
  CopyrightL&W Computer Services Ltd.                                      
     AuthorIan Day                                                         
  */
METHOD Init(nTop AS INT, nLeft AS INT, nBottom AS INT, nRight AS INT, aArray AS ARRAY),;
           (nTop, nLeft, nBottom, nRight)

   LOCAL oColumn

   ::aArray   := aArray
   ::nElement := 1

   ::SkipBlock     := {|nToMove| ::Skipper(nToMove) }
   ::GoTopBlock    := {|| ::nElement := 1 }
   ::GoBottomBlock := {|| ::nElement := Len(::aArray) }

   ::HeadSep := ""
   ::ColSep  := " "
   ::FootSep := ""

   ::TopArrow := {0, 0}
   ::BottomArrow := {MaxRow(), 0}

   ::lSearchable := .F.
   ::cSearch := ""

   ::bFinder := {|aPart| Upper(Left(aPart, Len(::cSearch))) == ::cSearch }

RETURN (self)

/*Ŀ
     MethodSkipper(<nToMove>) --> <nMoved>                                 
    PurposeSkipper block method for abBrowse                               
  */
METHOD Skipper(nToMove)

   LOCAL nLen

   IF nToMove < 0
      IF ::nElement + nToMove < 1
         nToMove := -(::nElement - 1)
         ::nElement := 1
      ELSE
         ::nElement += nToMove
      ENDIF
   ELSE
      nLen := Len(::aArray)
      IF ::nElement + nToMove > nLen
         nToMove := nLen - ::nElement
         ::nElement := nLen
      ELSE
         ::nElement += nToMove
      ENDIF
   ENDIF

RETURN (nToMove)

/*Ŀ
  Method   DisplayArrows()                                                 
  Purpose                                                                  
  Date-Time15-11-94 06:07pm                                                
  CopyrightDark Black Software Ltd.             Date updated: 15-11-94     
  Author   Ian 'DrDebug' Day                    Time updated: 06:07:17pm   
  */
METHOD DisplayArrows()

   LOCAL nFirstVisible := (::nElement - ::RowPos) + 1

   IF nFirstVisible > 1
      @ ::TopArrow[1], ::TopArrow[2] SAY "" COLOR "W+/B"
   ELSE
      @ ::TopArrow[1], ::TopArrow[2] SAY " " COLOR "W+/B"
   ENDIF

   IF nFirstVisible + ::RowCount - 1 < Len(::aArray)
      @ ::BottomArrow[1], ::BottomArrow[2] SAY "" COLOR "W+/B"
   ELSE
      @ ::BottomArrow[1], ::BottomArrow[2] SAY " " COLOR "W+/B"
   ENDIF

   @:DoCursor()

RETURN (self)

/*Ŀ
  Method   ForceStable()                                                   
  Purpose  Stabilize and sort cursor out                                   
  */
METHOD ForceStable()

   IF !::Stable
      ::Super:ForceStable()

      ::nCursorRow := Row()
      ::nCursorCol := Col()
   ENDIF

   @:DoCursor()

RETURN (self)

/*Ŀ
  Method   Stabilize()                                                     
  Purpose  Stabilize once and do cursor if stable                          
  */
METHOD Stabilize()

   IF !::Stable
      IF ::Super:Stabilize()
         ::nCursorRow := Row()
         ::nCursorCol := Col()
      ENDIF
   ENDIF

   @:DoCursor()

RETURN (self)

/*Ŀ
  Method   Handler()                                                       
  Purpose  Handle the keyboard input                                       
  */
METHOD Handler(nKey AS INT)

   LOCAL cKey, nPos

   DO CASE
      CASE nKey == K_UP
         ::Up()
         ::cSearch := ""

      CASE nKey == K_DOWN
         ::Down()
         ::cSearch := ""

      CASE nKey == K_LEFT
         ::Left()
         ::cSearch := ""

      CASE nKey == K_RIGHT
         ::Right()
         ::cSearch := ""

      CASE nKey == K_HOME
         IF ::ColCount() == 1
            ::GoTop()
         ELSE
            ::Home()
         ENDIF
         ::cSearch := ""

      CASE nKey == K_CTRL_PGUP
         ::GoTop()
         ::cSearch := ""

      CASE nKey == K_END
         IF ::ColCount() == 1
            ::GoBottom()
         ELSE
            ::End()
         ENDIF
         ::cSearch := ""

      CASE nKey == K_CTRL_PGDN
         ::GoBottom()
         ::cSearch := ""

      CASE nKey == K_PGDN
         ::PageDown()
         ::cSearch := ""

      CASE nKey == K_PGUP
         ::PageUp()
         ::cSearch := ""

      CASE nKey == K_BS
         IF ::lSearchable
            ::cSearch := Left(::cSearch, Len(::cSearch) - 1)
            @:DoCursor()
         ENDIF

      OTHERWISE
         IF ::lSearchable
            IF nKey >= Asc(' ') .AND. nKey <= Asc('z')
               cKey := Upper(Chr(nKey))

               ::cSearch += cKey

               IF (nPos := aScan(::aArray, ::bFinder, ::nElement)) == 0
                  IF (nPos := aScan(::aArray, ::bFinder)) == 0
                     ::cSearch := Left(::cSearch, Len(::cSearch) - 1)
                  ELSE
                     ::nElement := nPos
                     @:Settle()
                  ENDIF
               ELSE
                  ::nElement := nPos
                  @:Settle()
               ENDIF
            ENDIF
         ENDIF

   ENDCASE

RETURN (self)

/*Ŀ
  Method   Settle                                                          
  Purpose  Set the position in the abBrowse                                
  */
METHOD Settle

   LOCAL nRows, nLen, nElement, oBrowse

   nRows    := ::RowCount()
   nLen     := Len(::aArray)
   nElement := ::nElement

   IF nElement < 1
      ::nElement := nElement := 1
   ELSEIF nElement > nLen
      ::nElement := nElement := nLen
   ENDIF

   ::RefreshAll()

   IF nLen <= nRows
      ::RowPos := nElement
      ::nElement := 1
   ELSEIF nElement < nRows / 2
      ::RowPos  := nElement
      ::nElement := 1
   ELSEIF nElement + (nRows / 2) >= nLen
      ::RowPos  := nRows - (nLen - nElement)
      ::nElement := nElement - ::RowPos + 1
   ELSE
      ::RowPos  := nRows / 2 + 1
      ::nElement := nElement - ::RowPos + 1
   ENDIF

   @:ForceStable()

RETURN (self)

/*Ŀ
  Method   DoCursor                                                        
  Purpose  Make sure the cursor is pointing at the right character         
  */
METHOD DoCursor

   IF ::lSearchable
      IF Len(::cSearch) > 0
         SetPos(::nCursorRow, ::nCursorCol + Len(::cSearch))
         SetCursor(1)
      ELSE
         SetCursor(0)
      ENDIF
   ENDIF

RETURN (self)
