#include "mrdebug.ch"

/*Ŀ
  Function AssertTest                                                      
  Purpose  Test the assertion checking of MrDebug                          
  */
FUNCTION AssertTest(xParameter AS INT)

/* Ŀ
    We've already shown the first capability of Assertion check       
    (assuming that you havn't turned it off from the assertion        
    menu).  As you can see the program has stopped on the             
    function line above.  If you look in the assertion window         
    (Alt-Minus to switch to it) it will tell you the the 1st          
    parameter is not what was expected by this function, an           
    INTeger in this case.                                             
                                                                      
    A Trace file (MRDDEMO.TRC in this demo) will be created by        
    default with the same information in it.  So you can browse       
    it whilst seeing to your source code.                             
                                                                      
    The key advantage over this assertion system is that you are      
    still in the program.  It hasn't fallen over with a run-time      
    error.  It hasn't actually caused any problems yet.  But, if      
    let to run it's course the program would probably fall over       
    (except in this case were we're not going to use it...)           
                                                                      
    So, what are your options?  With any normal assertion system,     
    your only option is probably to press enter on the Quit button    
    of the Alert box and start on the very short journey to DOS.      
               not very promising...                                  
                                                                      
    With MrDebug, you have a few more.                                
                                                                      
       1) Change the value of the variable right now, so your program 
          can continue to run and you save yourself a compile & link. 
                                                                      
       2) You could also make a note at this point with the MrDebug   
          notes option.  Just press Shift-Alt-N to bring up an input  
          field.                                                      
                                                                      
       3) You can view the call stack, to see why the program has     
          passed such a parameter to this routine.                    
                                                                      
                                                                      
    OK, lets show you another example.  Imagine your small routine    
    which builds a tBrowse for the current workarea, which returns    
    .T. if it succeeds.  Ah, but that means we have to pass out       
    object variable by reference to the builder...  Mmmm, and we      
    always forget to do that...                                       
                                                                      
    So, what do you do?  Simple.  You use the ASREF argument for the  
    function declaration to tell the assertion system that the        
    parameter must be passed by reference.  Unfortunately, in this    
    case, you cannot adjust the value to a referenced value.  But,    
    that aside, the warning that it has happened could be enough to   
    save you hours of work!  I know I've done it myself.  This        
    example isn't particulaly effective, as it is just a simple one   
    level call, but when you've got 10 levels of objects and          
    arrays.... <g>  Anyway, enough prattle...  Press F5 now           
   */

   LOCAL oBrowse

   IF !FILE("DATA1.DBF")
      Data1Create()
   ELSE
      USE Data1 NEW VIA "DBFNTX" SHARED READONLY
   ENDIF

   IF MakeBrowse(oBrowse)
      // Use the browse
   ENDIF

   dbCloseArea()

   AnotherTest()

RETURN (NIL)

/*Ŀ
  Function MakeBrowse()                                                    
  Purpose  Make browse for workarea                                        
  */
STATIC FUNCTION MakeBrowse(oBrowse ASREF USUAL)

/* Ŀ
    OK, see the point.  We thought it would save time, so we put it in. 
    Press F5 now, to see the next example.                              
   */

   LOCAL lRet, nLoop, oColumn

   IF fCount() > 0
      oBrowse := TBrowseDB(0, 0, MaxRow(), MaxCol())

  /*  Ŀ
       This assignment will get lost at the end of the function as we've 
       neglected to pass by reference                                    
      */

      FOR nLoop := 1 TO fCount()
         oColumn := TBColumnNew(FieldName(nLoop), FieldWBlock(FieldName(nLoop), Select()))
         oBrowse:AddColumn(oColumn)
      NEXT nLoop
   ENDIF

RETURN (lRet) AS LOGIC

/*Ŀ
   <g> we forgot to setup the lRet value, so it's NIL istead of logical,    
   so an assertion check on your return values will weed out any little     
   bugs like this.  If we proceed now, the program will fall over, unless   
   of course you enter 'lRet := .F.' in the command window, and then you're 
   able to continue                                                         
  */




/*Ŀ
  Function AnotherTest()                                                   
  Purpose  Another test                                                    
  */
STATIC FUNCTION AnotherTest()

/* Ŀ
    We can also ASSERT expressions as well.  It wouldn'y be any 
    good if you couldn't...  Press F5 to see.                   
   */

   LOCAL aArray := GetStuff()

   ASSERT Len(aArray) > 100

/* Ŀ
    Well, we didn't get far!  Some demo program...  So, what     
    does the assertion window tell us?  Well, nothing more       
    than verbosely echoing the above expression into it.  Along  
    with the line number etc.  But, if you include more sensible 
    assertions in your code, you can add to the stability by     
    pointing out potential problems before they get to the       
    client's PC.                                                 
                                                                 
    This is probably a good point to show a small side feature,  
    now that you have several items in the assert window.  Just  
    switch to the assert window and select an item.  Press       
    <Space> and the source code window will update to show you   
    that piece of code.  If you press <Enter> instead, you will  
    be left in the source code window.                           
   */

RETURN (NIL)

/*Ŀ
  Function GetStuff()                                                      
  Purpose  Get some bits                                                   
  */
STATIC FUNCTION GetStuff()

   LOCAL aRet := Directory("*.*"), nLoop

   FOR nLoop := 1 TO Len(aRet)
      aRet[nLoop] := aRet[nLoop][1]
   NEXT nLoop

RETURN (aRet) AS ARRAY
