#include "mrdebug.ch"

/*Ŀ
  Function CondBreak                                                       
  Purpose  Conditional break point example                                 
  */
FUNCTION CondBreak

   LOCAL oWindow, nLoop, nSecond

   AltD()

   SetColor("BG+/B")
   oWindow := Window():New(10, 40, "Conditional Break Example"):Display()

/* Ŀ
    For this example we have a FOR..NEXT loop that just prints the loop  
    counter each time round and a little bit of DBF access to slow it    
    down a fraction.  But, imagine that it is a much more complex        
    operation and you want to stop the program at a particular line, but 
    not until the 200th iteration.  What would you normally do?          
                                                                         
       Set a Trace Point?                                                
       These can be slow and clumsy in such situations especially when   
       you're going in and out of other routines.                        
                                                                         
       Stick and AltD() inside an IF statement in your code?  This takes 
       time, and if you haven't got a really fast machine/linker etc.    
       Then you could be wasting a lot of time for such a simple need.   
       And it's really taking the biscuit when you have to edit your     
       source, recompile and re-link, just to stop the program where you 
       want it!                                                          
                                                                         
    So, the alternative is a conditional break point.  Something that    
    C/ASM/Pascal programmers have enjoyed for years, but Clipper has     
    been sadly lacking.                                                  
                                                                         
    Just go the the '? nLoop' line shown below and press Ctrl-F9.  This  
    will bring up a entry field for the Break Expression.  Enter the     
    expression to stop for (ie: 'nLoop == 200').  And then just run the  
    program with F5.                                                     
   */

   IF !FILE("DATA1.DBF")
      Data1Create()
   ELSE
      USE Data1 NEW VIA "DBFNTX" SHARED READONLY
   ENDIF

   FOR nLoop := 1 TO 1000
      ? nLoop

      dbGoTop()

      DO WHILE !Eof()
         dbSkip(10)
      ENDDO

   NEXT nLoop

   dbCloseArea()

   oWindow:Close()

RETURN (NIL)
