#include "classy2.ch"
#include "mrdebug.ch"
#Include "Box.ch"

/*Ŀ
      ClassWindow                                                          
    PurposeA Window Class to creat windows on the screen. To give them     
           headers, titles and anything else we can think of               
  Date-Time14/05/93      00:28:41                                          
  CopyrightDark Black Software Limited                                     
  */
CREATE CLASS Window

EXPORT:
   VAR Cargo

   VAR cTitleColour, cBorderColour  TYPE CHARACTER

   VAR nHandle                      TYPE INT

   VAR cTitle                       TYPE CHARACTER

   VAR cTitleColour, cBorderColour  TYPE CHARACTER

   CLASS VAR aStack                 TYPE ARRAY
   MESSAGE aStack TO CLASS

   METHOD nTop, nLeft, nBottom, nRight
   METHOD _nTop, _nLeft, _nBottom, _nRight

   METHOD Init, Close, WhoAmI, Display, DispWindow, Title, Show, Hide,;
          CloseAll

   MESSAGE Select METHOD _Select

END CLASS

/*Ŀ
     Methodinit(nTop, nLeft, nHeight, nWidth, cTitle)                      
    PurposeThis method is used to set up all the vars for the object       
                                                                           
            nTop        : the top row, where the window will start         
            nLeft       : the left column, where the window will start     
            nHeight     : the number of rows that the window uses          
            nWidth      : the number of columns the window takes up        
            cTitleColor : the Title header color for the window            
            cBackColor  : the Window pane color                            
            cTitle      : the text to diaplay in the title bar             
            lCanBeSeen  : is the window being displayed                    
            aStack      : stack used to hold window number and position    
            lFocus      : is the window the output window                  
                                                                           
                                                                           
  Date-Time14/05/93      00:28:45                                          
  CopyrightDark Black Software Limited                                     
     AuthorDarren Lancaster                                                
  */
METHOD init(nTop AS INT, nLeft AS INT, nHeight AS USUAL, nWidth AS USUAL, cTitle AS USUAL)

   LOCAL nOldWin := WSelect()

   IF nTop != NIL .AND. nLeft != NIL .AND. ValType(nHeight) != "N"
      cTitle  := nHeight
      nHeight := nTop
      nWidth  := nLeft
      nTop    := NIL
      nLeft   := NIL
   ENDIF

   IF nTop == NIL
      nTop    := Int(MaxRow(.T.) / 2 - (nHeight + 4) / 2)
   ENDIF

   IF nLeft == NIL
      nLeft   := Int(MaxCol(.T.) / 2 - (nWidth + 2) / 2)
   ENDIF

   ASSERT nTop    AS INT
   ASSERT nLeft   AS INT
   ASSERT nHeight AS INT
   ASSERT nWidth  AS INT
   ASSERT cTitle  AS STRING

   IF nWidth < 1
      nWidth := 1
   ENDIF
   IF nHeight < 1
      nHeight := 1
   ENDIF

   ::cTitle := cTitle
   ::cBorderColour := "GR+/B"
   ::cTitleColour := "GR+/B"

   ::nHandle := WOpen(nTop, nLeft, nTop + nHeight + 3, nLeft + nWidth + 1, .F., .F.)
   WFormat(3, 1, 1, 1)
   SetColor(::cBorderColour)
   Scroll()

   IF VALTYPE(::aStack) != "A"
      ::Class:aStack := {}
   ENDIF

   aAdd(::Class:aStack, self)

RETURN (self) AS OBJECT

/*Ŀ
     MethodClose() --> self                                                
    PurposeTo remove the window from the stack and the display             
  */
METHOD Close()

   LOCAL nPos := aScan(::aStack, {|oPart| oPart:nHandle == WSelect() })

   ::aStack:DeleteAt(nPos)

   WClose(::nHandle)

RETURN (self) AS OBJECT

/*Ŀ
     MethodWhoAmI() --> <nWindowNumber>                                    
    PurposeWhich window am I?                                              
  */
METHOD WhoAmI()
RETURN (WSelect()) AS INT

/*Ŀ
     MethodDisplay([<lFocus>]) --> self                                    
    Purposeto draw a new window on the screen. Save the background first   
           draw the box, draw the title                                    
  Date-Time14/05/93      00:28:55                                          
  CopyrightDark Black Software Limited                                     
     AuthorDarren Lancaster                                                
  */
METHOD Display(lFocus)

   WSelect(::nHandle)
   @:DispWindow()
   @:Title()
   WVisible(.T., ::nHandle)

RETURN (self) AS OBJECT

/*Ŀ
  Method   DispWindow()                                                    
  Purpose  Display the window                                              
  */
METHOD DispWindow()

   WFormat(-3, -1, -1, -1)

   WSelect(::nHandle)
   DispBox(0, 0, MaxRow(), MaxCol(), B_DOUBLE, ::cBorderColour)
   @ 2, 0 SAY "" COLOR ::cBorderColour
   @ 2, MaxCol() SAY "" COLOR ::cBorderColour

   DispBox(2, 1, 2, MaxCol() - 1, B_DOUBLE, ::cBorderColour)

   WFormat(3, 1, 1, 1)

RETURN (self) AS OBJECT

/*Ŀ
     MethodTitle() --> self                                                
    PurposeDisplay the Title                                               
  */
METHOD Title()

   WSelect(::nHandle)
   WFormat(-2, 0, 0, 0)
   @ 0, 0 SAY PadC(::cTitle, MaxCol() + 1) COLOR ::cTitleColour
   WFormat(2, 0, 0, 0)

RETURN (self) AS OBJECT

/*Ŀ
     Methodshow                                                            
    Purposeto show a window that has been hidden                           
  */
METHOD Show()

   WVisible(.T., ::nHandle)

RETURN (self) AS OBJECT

/*Ŀ
  Method   Select()                                                        
  Purpose  Select this window                                              
  */
METHOD _Select()

   WSelect(::nHandle)

RETURN (self) AS OBJECT

/*Ŀ
     MethodHide                                                            
    Purposeto restore the background and hide the window                   
  Date-Time14/05/93      00:28:59                                          
  CopyrightDark Black Software Limited                                     
     AuthorDarren Lancaster                                                
  */
METHOD Hide()

   WVisible(.F., ::nHandle)

RETURN (self) AS OBJECT

/*Ŀ
  Method   CloseAll                                                        
  Purpose  Close all windows                                               
  */
METHOD CloseAll

   aEval(::aStack, {|oPart| WClose(oPart:nHandle) })
   ::Class:aStack := {}

RETURN (self)

/* These are access/assign methods which get set values from the windows */

/*Ŀ
  Method   nTop                                                            
  Purpose  Get position of top of window                                   
  */
METHOD nTop
RETURN (WRow(::nHandle)) AS INT

/*Ŀ
  Method   nleft                                                           
  Purpose  Get position of left of window                                  
  */
METHOD nLeft
RETURN (WCol(::nHandle)) AS INT

/*Ŀ
  Method   nBottom                                                         
  Purpose  Get position of bottom of window                                
  */
METHOD nBottom
RETURN (WLastRow(::nHandle)) AS INT

/*Ŀ
  Method   nRight                                                          
  Purpose  Get position of right of window                                 
  */
METHOD nRight
RETURN (WLastCol(::nHandle)) AS INT

/*Ŀ
  Method   _nTop                                                           
  Purpose  Set position of top of window                                   
  */
METHOD _nTop(nNewRow AS INT)
RETURN (WMove(nNewRow, WCol(::nHandle), ::nHandle))

/*Ŀ
  Method   nleft                                                           
  Purpose  Get position of left of window                                  
  */
METHOD _nLeft(nNewCol AS INT)
RETURN (WMove(WRow(::nHandle), nNewCol, ::nHandle))

/*Ŀ
  Method   nBottom                                                         
  Purpose  Get position of bottom of window                                
  */
METHOD _nBottom(nNewRow AS INT)
RETURN (@:nBottom()) AS INT

/*Ŀ
  Method   nRight                                                          
  Purpose  Get position of right of window                                 
  */
METHOD _nRight(nNewCol AS INT)
RETURN (@:nRight()) AS INT
