/* Program Ŀ
      File Name: _MRDERSY.PRG                                               
      Copyright: 1993 by L&W Computer Services Ltd.                         
  Ĵ
   This errorsys should be used, rather than the default clipper one, if    
   you intend to use MrDebug with large screen modes.  This is because of a 
   bug in clipper which garbles the application screen if it is not the     
   same size as the physical screen.  This is unfortunate as we know that   
   many of you will have developed your own ErrorSys replacements.  The     
   only difference with this one is the we do not use STDOUT to display the 
   errors.  So if you wish to use your own, then simply replace STDOUT with 
   a normal print statement (?).                                            
                                                                            
   This is only necessary if you are debugging in large screen modes        
   (ie: greater than 80 columns) and you have the application screen in an  
   80x25 window.                                                            
                                                                            
   You also get more informative errors from this one.  So instead of just  
   'Argument error' we give you something like:  'Argument error: 1 + .T.'  
  Ĵ
   If you are using Class(y) then you should replace the calls to           
   ProcName() with calls to MethodName() instead, this will give you a more 
   informative call stack trace when your application heads for troubled    
   times.                                                                   
  */

#include "error.ch"
#include "mrdebug.ch"

/*Ŀ
   FunctionErrorSys() --> NIL                                              
    PurposeReplacement Default error handler                               
           Set's up the default error handling function.  This is called   
           internally by Clipper, so no user intervention is required.     
  CopyrightL&W Computer Services Ltd.                                      
  */
FUNCTION ErrorSys

   ErrorBlock({|oError| MrDebugErr(oError) })

RETURN (NIL)

/*Ŀ
   FunctionMrDebugErr(<oError>) --> <xSomething>                           
    PurposeDefault Error handler                                           
  Descript.This first checks to see if we had a network error, a divide by 
           zero error or an Append lock failure.  If not, then control is  
           passed to the error message handling function.                  
  CopyrightL&W Computer Services Ltd.                                      
  */
STATIC FUNCTION MrDebugErr(oError)

   LOCAL oOld := ErrorBlock({|| QOUT("Error in error handler: " + ProcName(1) + "(" + lTrim(Str(ProcLine(1))) + ")"), __QUIT() }),;
         xRet

   MrdSaveErr(oError)      // Save the error Object

   // Network error? Yup, lets set NETERR() to inform the program
   IF oError:GenCode == EG_OPEN .AND. oError:OsCode == 32 .AND. oError:CanDefault
      NetErr(.T.)
      ErrorBlock(oOld)
      RETURN (.F.)      // Let's not do it again!
   ENDIF

   // This handles divide by zero errors, we return 0
   IF oError:GenCode == EG_ZERODIV
      ErrorBlock(oOld)
      RETURN (0)
   ENDIF

   // Append new record, oops where's the lock gone?
   IF oError:GenCode == EG_APPENDLOCK .AND. oError:CanDefault
      NetErr(.T.)
      ErrorBlock(oOld)
      RETURN (.F.)      // Don't try again
   ENDIF

   xRet := OtherStuff(oError) // Well, that's the basics.  Let's see what's gone wrong

   ErrorBlock(oOld)

RETURN (xRet)

/*Ŀ
   FunctionOtherstuff(<oError>) --> NIL                                    
    PurposeGive a decent error message                                     
  Descript.This handles the calling of ErrorMessage() and controls the     
           display of Alert() and the printing of the error message to the 
           screen.                                                         
  CopyrightL&W Computer Services Ltd.                                      
  */
STATIC FUNCTION OtherStuff(oError)

   LOCAL cMessage,;        // The error message
         nLoop,;           // Just a loop
         nStart := 3,;     // Where to start the call stack from
         aOptions,;        // For Alert()
         nAnswer           // From Alert()

   SET PRINTER OFF      // Turn the printer off
   SET CONSOLE ON       // Turn the screen on

   DO WHILE DispCount() > 0
      DispEnd()         // We'd like to see the error and what's on our screen at the moment
   ENDDO

   cMessage := ErrorMessage(oError)

   aOptions := {"Quit"}

   IF oError:CanRetry
      aAdd(aOptions, "Retry")
   ENDIF

   IF oError:CanDefault
      aAdd(aOptions, "Default")
   ENDIF

   nAnswer := 0

   DO WHILE nAnswer == 0
      nAnswer := Alert(cMessage, aOptions)         // Give the message
                                //  Hopefully fixes alert problem
   ENDDO

   IF nAnswer > 0
      IF "RET" $ Upper(aOptions[nAnswer])
         RETURN (.T.)
      ELSEIF "DEF" $ Upper(aOptions[nAnswer])
         RETURN (.F.)
      ENDIF
   ENDIF

   SetColor("")      // Reset colours to default in case we have silly ones

   DO WHILE .T.      // Break the message into lines
      IF ";" $ cMessage
         ? Left(cMessage, At(";", cMessage) - 1)
         cMessage := SubStr(cMessage, At(";", cMessage) + 1)
      ELSE
         ? cMessage
         EXIT
      ENDIF
   ENDDO

   ?

   IF Right(cMessage, 1) == ";"
      cMessage := Left(cMessage, Len(cMessage) - 1)
   ENDIF

   IF "LOCKERRHAN" $ ProcName(nStart)     // This is always here, mostly
      nStart += 2
   ENDIF

   nLoop := nStart
   DO WHILE !Empty(ProcName(nLoop))
      ? "Called from", Trim(ProcName(nLoop)) + "(" + lTrim(Str(ProcLine(nLoop))) + ")"
      nLoop++
   ENDDO

   ErrorLevel(1)

   QUIT        // Awwww, going so soon?

RETURN (NIL)

/*Ŀ
   FunctionErrorMessage(<oError>) --> NIL                                  
    PurposeGet error message                                               
  Descript.Handles the redamentary part of the error message.              
              (See ErrorDesc() for more)                                   
  CopyrightL&W Computer Services Ltd.                                      
  */
STATIC FUNCTION ErrorMessage(oError)

   LOCAL cMessage

   cMessage := {"Who cares?", "Warning", "Error", "Disaster"}[If(oError:Severity >= 0 .AND. oError:Severity <= 3, oError:Severity + 1, 3)]

   cMessage += "  "

   IF ValType(oError:SubSystem) == "C"
      cMessage += oError:SubSystem
   ELSE
      cMessage += "???"
   ENDIF

   IF ValType(oError:SubCode) == "N"
      cMessage += "/" + lTrim(Str(oError:SubCode))
   ELSE
      cMessage += "/???"
   ENDIF

   cMessage += ";"

   // add error description if available
   IF ValType(oError:Description) == "C"
      cMessage += oError:Description
   ENDIF

   cMessage += (";" + ErrorDesc(oError))

RETURN (cMessage)
